# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .llm_as_judge_model_parameters import LlmAsJudgeModelParameters
import typing
from .llm_as_judge_message import LlmAsJudgeMessage
import typing_extensions
from .llm_as_judge_output_schema import LlmAsJudgeOutputSchema
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class LlmAsJudgeCode(UniversalBaseModel):
    model: LlmAsJudgeModelParameters
    messages: typing.List[LlmAsJudgeMessage]
    variables: typing.Dict[str, str]
    schema_: typing_extensions.Annotated[
        typing.List[LlmAsJudgeOutputSchema], FieldMetadata(alias="schema")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
