# This file was auto-generated by Fern from our API Definition.

import typing

AssistantMessageRole = typing.Union[
    typing.Literal["system", "user", "assistant", "tool", "function"], typing.Any
]
