# This file was auto-generated by Fern from our API Definition.

import typing

ColumnCompareTypesItem = typing.Union[
    typing.Literal["string", "number", "object", "boolean", "array", "null"], typing.Any
]
